/* Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "ble_hrs.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_l2cap.h"
#include "ble_srv_common.h"
#include "app_util.h"
#include "nrf_soc.h"
uint8_t data_desc[10] = "accel_data";


#define OPCODE_LENGTH  1                                                    /**< Length of opcode inside Heart Rate Measurement packet. */
#define HANDLE_LENGTH  2                                                    /**< Length of handle inside Heart Rate Measurement packet. */
#define MAX_ACCEL_LEN    (BLE_L2CAP_MTU_DEF - OPCODE_LENGTH - HANDLE_LENGTH)  /**< Maximum size of a transmitted Heart Rate Measurement. */

#define INITIAL_VALUE_HRM                       0                           /**< Initial Heart Rate Measurement value. */

// Heart Rate Measurement flag bits
#define ACCEL_FLAG_MASK_HR_VALUE_16BIT            (0x01 << 0)                 /**< Heart Rate Value Format bit. */
#define ACCEL_FLAG_MASK_SENSOR_CONTACT_DETECTED   (0x01 << 1)                 /**< Sensor Contact Detected bit. */
#define ACCEL_FLAG_MASK_SENSOR_CONTACT_SUPPORTED  (0x01 << 2)                 /**< Sensor Contact Supported bit. */
#define ACCEL_FLAG_MASK_EXPENDED_ENERGY_INCLUDED  (0x01 << 3)                 /**< Energy Expended Status bit. Feature Not Supported */
#define ACCEL_FLAG_MASK_RR_INTERVAL_INCLUDED      (0x01 << 4)                 /**< RR-Interval bit. */

//#define BLE_UUID_ACCEL_SERVICE                          0x0001
//#define BLE_UUID_ACCEL_DATA_CHARACTERISTIC              0x0002
//#define BLE_UUID_ACCEL_SETTING_CHARACTERISTIC           0x0003
//#define BLE_UUID_ACCEL_LOCATION_CHARACTERISTIC          0x0004
//#define BLE_UUID_ACCEL_DELETE_CHARACTERISTIC			      0x0005
//#define BLE_UUID_ACCEL_ACTIVITY_CHARACTERISTIC 				  0x0006
//#define BLE_UUID_ACCEL_POLL_CHARACTERISTIC         		  0x0007
//#define BLE_UUID_ACCEL_DFU_CHARACTERISTIC               0x0008


typedef struct ble_acc_s
{
    uint8_t                  uuid_type;                           /**< UUID type for Nordic UART Service Base UUID. */
} ble_acc_t;


/**@brief Function for handling the Connect event.
 *
 * @param[in]   p_hrs       Heart Rate Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_accel_t * p_accel, ble_evt_t * p_ble_evt)
{
    p_accel->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}


/**@brief Function for handling the Disconnect event.
 *
 * @param[in]   p_hrs       Heart Rate Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_accel_t * p_accel, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_accel->conn_handle = BLE_CONN_HANDLE_INVALID;
}


/**@brief Function for handling write events to the Heart Rate Measurement characteristic.
 *
 * @param[in]   p_hrs         Heart Rate Service structure.
 * @param[in]   p_evt_write   Write event received from the BLE stack.
 */
static void on_accel_cccd_write(ble_accel_t * p_accel, ble_gatts_evt_write_t * p_evt_write)
{
    if (p_evt_write->len == 2)
    {
        // CCCD written, update notification state
        if (p_accel->evt_handler != NULL)
        {
            ble_accel_evt_t evt;
            
            if (ble_srv_is_notification_enabled(p_evt_write->data))
            {
                evt.evt_type = BLE_ACCEL_EVT_NOTIFICATION_ENABLED;
            }
            else
            {
                evt.evt_type = BLE_ACCEL_EVT_NOTIFICATION_DISABLED;
            }
            
            p_accel->evt_handler(p_accel, &evt);
        }
    }
}


/**@brief Function for handling the Write event.
 *
 * @param[in]   p_hrs       Heart Rate Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_accel_t * p_accel, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;
	
	/* if(p_evt_write->handle == 0x8248)
			{		
				sd_power_gpregret_set(1);
				sd_nvic_SystemReset();
			}
	*/
    if (p_evt_write->handle == p_accel->accel_data_handles.cccd_handle)
    {
        on_accel_cccd_write(p_accel, p_evt_write);
    }
		
		if (p_evt_write->handle == p_accel->accel_setting_handles.cccd_handle)
    {
        on_accel_cccd_write(p_accel, p_evt_write);
    }
		if (p_evt_write->handle == p_accel->accel_delete_handles.cccd_handle)
    {
        on_accel_cccd_write(p_accel, p_evt_write);
    }
}


void ble_accel_on_ble_evt(ble_accel_t * p_accel, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_accel, p_ble_evt);
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_accel, p_ble_evt);
            break;
            
        case BLE_GATTS_EVT_WRITE:
            on_write(p_accel, p_ble_evt);
            break;
            
        default:
            break;
    }
}


/**@brief Function for encoding a Heart Rate Measurement.
 *
 * @param[in]   p_hrs              Heart Rate Service structure.
 * @param[in]   heart_rate         Measurement to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
/*
static uint8_t accel_data_encode(ble_accel_t * p_hrs, uint16_t accel_rate, uint8_t * p_encoded_buffer)
{
    uint8_t flags = 0;
    uint8_t len   = 1;
    int     i;
    
    // Set sensor contact related flags
    if (p_hrs->is_sensor_contact_supported)
    {
        flags |= ACCEL_FLAG_MASK_SENSOR_CONTACT_SUPPORTED;
    }
    if (p_hrs->is_sensor_contact_detected)
    {
        flags |= ACCEL_FLAG_MASK_SENSOR_CONTACT_DETECTED;
    }

    // Encode heart rate measurement
    if (accel_rate > 0xff)
    {
        flags |= ACCEL_FLAG_MASK_HR_VALUE_16BIT;
        len += uint16_encode(accel_rate, &p_encoded_buffer[len]);
    }
    else
    {
        p_encoded_buffer[len++] = (uint8_t)accel_rate;
    }
    
    // Encode rr_interval values
    if (p_hrs->rr_interval_count > 0)
    {
        flags |= HRM_FLAG_MASK_RR_INTERVAL_INCLUDED;
    }
    for (i = 0; i < p_hrs->rr_interval_count; i++)
    {
        if (len + sizeof(uint16_t) > MAX_HRM_LEN)
        {
            // Not all stored rr_interval values can fit into the encoded hrm,
            // move the remaining values to the start of the buffer.
            memmove(&p_hrs->rr_interval[0],
                    &p_hrs->rr_interval[i],
                    (p_hrs->rr_interval_count - i) * sizeof(uint16_t));
            break;
        }
        len += uint16_encode(p_hrs->rr_interval[i], &p_encoded_buffer[len]);
    }
    p_hrs->rr_interval_count -= i;

    // Add flags
    p_encoded_buffer[0] = flags;
    
    return len;
}
*/

/**@brief Function for adding the Heart Rate Measurement characteristic.
 *
 * @param[in]   p_hrs        Heart Rate Service structure.
 * @param[in]   p_hrs_init   Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */

static uint32_t accel_data_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
	
   // uint8_t             encoded_initial_hrm[MAX_ACCEL_LEN];
	
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_DATA_CHAR;
		
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_data_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
	//	char_md.char_props.read   = 1;
		char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
   // BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_DATA_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_data_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_data_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 6;							//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = 	0;						//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_data_handles);
																					 
	/*																				 
		BLE_UUID_BLE_ASSIGN(char_uuid, BLE_UUID_REPORT_REF_DESCR);
		
		memset(&attr_md, 0, sizeof(attr_md));
		
		 attr_md.read_perm = p_accel_init->accel_data_attr_md.read_perm;
     BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);

        attr_md.vloc       = BLE_GATTS_VLOC_STACK;
        attr_md.rd_auth    = 0;
        attr_md.wr_auth    = 0;
        attr_md.vlen       = 0;
        
 //       init_len = ble_srv_report_ref_encode(encoded_report_ref, p_bas_init->p_report_ref);
        
        memset(&attr_char_value, 0, sizeof(attr_char_value));

        attr_char_value.p_uuid       = &char_uuid;
        attr_char_value.p_attr_md    = &attr_md;
        attr_char_value.init_len     = 0;							//init_len;
        attr_char_value.init_offs    = 0;
        attr_char_value.max_len      = 20;								//attr_char_value.init_len;
        attr_char_value.p_value      = data_desc;										//encoded_report_ref;
        
        return sd_ble_gatts_descriptor_add(p_accel->accel_data_handles.value_handle,
                                               &attr_char_value,
                                               &p_accel->report_ref_handle);   */
       
   
		/*	 if (err_code != NRF_SUCCESS)
        {
            return err_code;
        }
	
																				 
																				 
																							 
		err_code = sd_ble_gatts_descriptor_add(p_accel->accel_data_handles.value_handle,
                                               &attr_char_value,
                                               &p_accel->report_ref_handle);   */
       
			}


static uint32_t accel_location_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
  //  uint8_t             encoded_initial_hrm[MAX_HRM_LEN];
    
	  char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_LOCATION_CHAR;
	
    memset(&cccd_md, 0, sizeof(cccd_md));

		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_location_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;   

    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify  = 1;
	//	char_md.char_props.write = 1;
		char_md.char_props.read   = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
 //   BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_LOCATION_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_location_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_location_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2;					//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = 	0;						//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_location_handles);
}

static uint32_t accel_setting_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
		uint8_t  config_data[2] = {1, 16};
		
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
  //  uint8_t             encoded_initial_hrm[MAX_ACCEL_LEN];
	
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_SETTING_CHAR;
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_setting_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
		char_md.char_props.write = 1;
		char_md.char_props.read = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
  //  BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_SETTING_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_setting_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_setting_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2;						//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = config_data;						//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_setting_handles);
}


static uint32_t accel_delete_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
  //  uint8_t             encoded_initial_hrm[MAX_HRM_LEN];
    
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_DELETE_CHAR;
    memset(&char_md, 0, sizeof(char_md));
    
   	char_md.char_props.write = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;			//&cccd_md;
    char_md.p_sccd_md         = NULL;
    
  //  BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_DELETE_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_delete_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_delete_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 1;					//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 1;					//MAX_HRM_LEN;
    attr_char_value.p_value      = 0;				 //encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_delete_handles);
}



static uint32_t accel_activity_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
  //  uint8_t             encoded_initial_hrm[MAX_HRM_LEN];
    char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_ACTIVITY_CHAR;
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_activity_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
  //char_md.char_props.notify = 1;
		char_md.char_props.read 	= 1;
		char_md.char_props.write 	= 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
 //   BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_ACTIVITY_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_activity_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_activity_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 8; //hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = 0;	//m_cur_heart_rate;						//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_activity_handles);
}


static uint32_t accel_poll_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
 //   uint8_t             encoded_initial_hrm[MAX_ACCEL_LEN];
	
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_POLL_CHAR;
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_poll_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
 //   char_md.char_props.notify = 1;
		char_md.char_props.read   = 1;
		char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
 //   BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_POLL_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_poll_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_poll_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2;				//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = 0;			//	encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_poll_handles);
}


static uint32_t accel_dfu_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
 //   uint8_t             encoded_initial_hrm[MAX_ACCEL_LEN];
	
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_DFU_CHAR;
    
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_dfu_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
 // char_md.char_props.notify = 1;
//	char_md.char_props.read   = 1;
		char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
 //   BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_DFU_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_dfu_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_dfu_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 1;				//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_ACCEL_LEN;
    attr_char_value.p_value      = 0;			//	encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_dfu_handles);
}

static uint32_t accel_time_char_add(ble_accel_t *            p_accel,
                                                const ble_accel_init_t * p_accel_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          char_uuid;
    ble_gatts_attr_md_t attr_md;
 //   uint8_t             encoded_initial_hrm[MAX_ACCEL_LEN];
	
		char_uuid.type = p_accel->uuid_type;
    char_uuid.uuid = BLE_UUID_ACCELEROMETER_TIME_CHAR;
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_accel_init->accel_time_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
    
 // char_md.char_props.notify = 1;
		char_md.char_props.read   = 1;
		char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = 0;   //NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
 //   BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_ACCELEROMETER_TIME_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_accel_init->accel_time_attr_md.read_perm;
    attr_md.write_perm = p_accel_init->accel_time_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &char_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 4;				//hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 4;				//MAX_ACCEL_LEN;
    attr_char_value.p_value      = 0;			//	encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_accel->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_accel->accel_time_handles);

}




/**@brief Function for adding the Body Sensor Location characteristic.
 *
 * @param[in]   p_hrs        Heart Rate Service structure.
 * @param[in]   p_hrs_init   Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
 
 /*
static uint32_t body_sensor_location_char_add(ble_hrs_t * p_hrs, const ble_hrs_init_t * p_hrs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.read  = 1;
	char_md.char_props.notify  = 1;
    char_md.p_char_user_desc = NULL;
    char_md.p_char_pf        = NULL;
    char_md.p_user_desc_md   = NULL;
    char_md.p_cccd_md        = NULL;
    char_md.p_sccd_md        = NULL;
    
    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_BODY_SENSOR_LOCATION_CHAR);
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_hrs_init->hrs_bsl_attr_md.read_perm;
    attr_md.write_perm = p_hrs_init->hrs_bsl_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = p_hrs_init->p_body_sensor_location;
    
    return sd_ble_gatts_characteristic_add(p_hrs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_hrs->bsl_handles);
}

*/

/*static uint32_t heart_rate_check_char_add(ble_hrs_t *            p_hrs,
                                                const ble_hrs_init_t * p_hrs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    uint8_t             encoded_initial_hrm[MAX_HRM_LEN];
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_hrs_init->hrs_check_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
		char_md.char_props.write = 1;
		char_md.char_props.read = 1;
    char_md.p_char_user_desc  = 0000;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_REPORT_CHAR);
		
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_hrs_init->hrs_check_attr_md.read_perm;
    attr_md.write_perm = p_hrs_init->hrs_check_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2; //hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_HRM_LEN;
    attr_char_value.p_value      = 0;     //new_data; //get_data();//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_hrs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_hrs->check_handles);
}



static uint32_t heart_rate_sample_char_add(ble_hrs_t *            p_hrs,
                                                const ble_hrs_init_t * p_hrs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    uint8_t             encoded_initial_hrm[MAX_HRM_LEN];
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.write_perm = p_hrs_init->hrs_sample_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
		char_md.char_props.write = 1;
		char_md.char_props.read = 1;
    char_md.p_char_user_desc  = 0000;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_MODEL_NUMBER_STRING_CHAR);
		
    
    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_hrs_init->hrs_sample_attr_md.read_perm;
    attr_md.write_perm = p_hrs_init->hrs_sample_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    
    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2; //hrm_encode(p_hrs, INITIAL_VALUE_HRM, encoded_initial_hrm);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = MAX_HRM_LEN;
    attr_char_value.p_value      = 0;//encoded_initial_hrm;
    
    return sd_ble_gatts_characteristic_add(p_hrs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_hrs->sample_handles);
}
*/

uint32_t ble_accel_init(ble_accel_t * p_accel, const ble_accel_init_t * p_accel_init)
{
    uint32_t   err_code;
    ble_uuid_t  service_uuid;
	
	//Assign custom UUID
    const ble_uuid128_t base_uuid128 =
    {
       {
           0x4F, 0x64, 0x05, 0x28, 0xA9, 0x9F, 0xB2, 0x9D,
           0x06, 0xF6, 0x3C, 0xC3, 0x00, 0x00, 0x7E, 0x7A
       }
    };

		service_uuid.uuid = BLE_UUID_ACCEL_SERVICE;

    err_code = sd_ble_uuid_vs_add(&base_uuid128, &(service_uuid.type));
    if (err_code != NRF_SUCCESS)
    {
       APP_ERROR_CHECK(err_code);
    }
	
    // Initialize service structure
    p_accel->evt_handler                 = p_accel_init->evt_handler;
    p_accel->is_sensor_contact_supported = p_accel_init->is_sensor_contact_supported;
    p_accel->conn_handle                 = BLE_CONN_HANDLE_INVALID;
    p_accel->is_sensor_contact_detected  = false;
   //  p_hrs->rr_interval_count          = 0;
    p_accel->uuid_type                   = service_uuid.type;
		
    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &service_uuid, &p_accel->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
  
		// Accelerometer data characteristic
    err_code = accel_data_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
	
		err_code = accel_location_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }  
		
		
		err_code = accel_setting_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
		
		err_code = accel_delete_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
		
		err_code = accel_activity_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
		err_code = accel_poll_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;   
    }
		err_code = accel_dfu_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
	
    
		
		err_code = accel_time_char_add(p_accel, p_accel_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }      
	/*	err_code = heart_rate_4_char_add(p_hrs, p_hrs_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    
    if (p_hrs_init->p_body_sensor_location != NULL)
    {
        // Add body sensor location characteristic
        err_code = body_sensor_location_char_add(p_hrs, p_hrs_init);
        if (err_code != NRF_SUCCESS)
        {
            return err_code;
        }
    }
    

err_code = heart_rate_sample_char_add(p_hrs, p_hrs_init);
		if (err_code != NRF_SUCCESS)
        {
            return err_code;
        }
	err_code = heart_rate_check_char_add(p_hrs, p_hrs_init);		
				if (err_code != NRF_SUCCESS)
        {
            return err_code;
        }*/
				
    return NRF_SUCCESS;
}



uint32_t ble_accel_data_send(ble_accel_t * p_accel,uint16_t length, uint8_t *accel_rate)
{
    uint32_t err_code;
	    
    // Send value if connected and notifying
    if (p_accel->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
     //   uint8_t                encoded_hrm[MAX_HRM_LEN];
        uint16_t               len;
				uint16_t               hvx_len;
       ble_gatts_hvx_params_t  hvx_params;
			 ble_gatts_evt_write_t   write_params;
			
			 len     = length;
		
       hvx_len = len; //len1;
		   /*memset(&write_params, 0, sizeof(write_params));
		   write_params.handle   = p_accel->accel_data_handles.value_handle;
       hvx_params.type       = BLE_GATT_HVX_NOTIFICATION;
		//   write_params.op     = BLE_GATT_OP_EXECUTE_WRITE_REQ;
		   write_params.offset   = 0;
       write_params.len      = hvx_len;
       write_params.data     = accel_rate;
       */

       memset(&hvx_params, 0, sizeof(hvx_params));
        
        hvx_params.handle   = p_accel->accel_data_handles.value_handle;
        hvx_params.type     = BLE_GATT_HVX_NOTIFICATION;
			//	hvx_params.type     = BLE_GATT_OP_EXECUTE_WRITE_REQ;
				hvx_params.offset   = 0;
        hvx_params.p_len    = &hvx_len;
        hvx_params.p_data   = 0;		//	accel_rate;
        
        err_code = sd_ble_gatts_hvx(p_accel->conn_handle, &hvx_params); 
			//	err_code = sd_ble_gatts_value_set(0x0017, 0, &len,heart_rate );
        if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }     
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;    
} 


uint32_t ble_accel_setting_send(ble_accel_t * p_accel,uint16_t length, uint16_t handle)
{
    uint32_t err_code;
	    
    // Send value if connected and notifying
    if (p_accel->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
     // uint8_t                encoded_hrm[MAX_HRM_LEN];
        uint16_t               len;
				uint16_t               hvx_len;
				ble_gatts_hvx_params_t  hvx_params;
				
			 len     = length;
		   hvx_len = len; //len;
	
       memset(&hvx_params, 0, sizeof(hvx_params));
        
        /*hvx_params.handle   = p_accel->accel_setting_handles.value_handle;
        hvx_params.type     = BLE_GATT_HVX_NOTIFICATION;
	 //	  hvx_params.type     = BLE_GATT_OP_EXECUTE_WRITE_REQ;
				hvx_params.offset   = 0;
        hvx_params.p_len    = &hvx_len;
        hvx_params.p_data   = handle;
        */
        err_code = sd_ble_gatts_hvx(p_accel->conn_handle, &hvx_params); 
	      if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }     
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;    
} 
/*
//---------------------------
uint32_t ble_hrs_heart_rate_measurement_send1(ble_hrs_t * p_hrs, uint8_t *heart_rate)
{
    uint32_t err_code;
       // Send value if connected and notifying
    if (p_hrs->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        uint8_t                encoded_hrm[MAX_HRM_LEN];
        uint16_t               len;
        uint16_t               hvx_len;
        ble_gatts_hvx_params_t hvx_params;
        
        len     = 1;//hrm_encode(p_hrs, *heart_rate, encoded_hrm);
        hvx_len = 1; //len;

        memset(&hvx_params, 0, sizeof(hvx_params));
        
        hvx_params.handle   = p_hrs->hrm1_handles.value_handle;
        hvx_params.type     = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset   = 0;
        hvx_params.p_len    = &hvx_len;
        hvx_params.p_data   = heart_rate;
        
        err_code = sd_ble_gatts_hvx(p_hrs->conn_handle, &hvx_params);
        if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}

//---------------------------------------------

uint32_t ble_hrs_heart_rate_measurement_send(ble_hrs_t * p_hrs, uint8_t *heart_rate)
{
    uint32_t err_code;
    
    // Send value if connected and notifying
    if (p_hrs->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        uint8_t                encoded_hrm[MAX_HRM_LEN];
        uint16_t               len;
        uint16_t               hvx_len;
        ble_gatts_hvx_params_t hvx_params;
        
        len     = 	20;		//hrm_encode(p_hrs, *heart_rate, encoded_hrm);
        hvx_len = 	20;	//len;

        memset(&hvx_params, 0, sizeof(hvx_params));
        
        hvx_params.handle   = p_hrs->hrm_handles.value_handle;
        hvx_params.type     = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset   = 0;
        hvx_params.p_len    = &hvx_len;
        hvx_params.p_data   = heart_rate;
        
        err_code = sd_ble_gatts_hvx(p_hrs->conn_handle, &hvx_params);
        if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}



void ble_hrs_rr_interval_add(ble_hrs_t * p_hrs, uint16_t rr_interval)
{
    if (p_hrs->rr_interval_count == BLE_HRS_MAX_BUFFERED_RR_INTERVALS)
    {
        // The rr_interval buffer is full, delete the oldest value
        memmove(&p_hrs->rr_interval[0],
                &p_hrs->rr_interval[1],
                (BLE_HRS_MAX_BUFFERED_RR_INTERVALS - 1) * sizeof(uint16_t));
        p_hrs->rr_interval_count--;
    }
    
    // Add new value
    p_hrs->rr_interval[p_hrs->rr_interval_count++] = rr_interval;
}


bool ble_hrs_rr_interval_buffer_is_full(ble_hrs_t * p_hrs)
{
    return (p_hrs->rr_interval_count == BLE_HRS_MAX_BUFFERED_RR_INTERVALS);
}


uint32_t ble_hrs_sensor_contact_supported_set(ble_hrs_t * p_hrs, bool is_sensor_contact_supported)
{
    // Check if we are connected to peer
    if (p_hrs->conn_handle == BLE_CONN_HANDLE_INVALID)
    {
        p_hrs->is_sensor_contact_supported = is_sensor_contact_supported;
        return NRF_SUCCESS;
    }
    else
    {
        return NRF_ERROR_INVALID_STATE;
    }
}


void ble_hrs_sensor_contact_detected_update(ble_hrs_t * p_hrs, bool is_sensor_contact_detected)
{
    p_hrs->is_sensor_contact_detected = is_sensor_contact_detected;
}


uint32_t ble_hrs_body_sensor_location_set(ble_hrs_t * p_hrs, uint8_t body_sensor_location)
{
    uint16_t len = sizeof(uint8_t);
    return sd_ble_gatts_value_set(p_hrs->bsl_handles.value_handle, 0, &len, &body_sensor_location);
}
*/